@@ -2,11 +2,11 @@
 **  Copyright (c) 2005, 2006 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-filter.c,v 1.76 2006/06/12 17:44:09 msk Exp $
+**  $Id: dkim-filter.c,v 1.79 2006/09/12 21:46:30 msk Exp $
 */
 
 #ifndef lint
-static char dkim_filter_c_id[] = "@(#)$Id: dkim-filter.c,v 1.76 2006/06/12 17:44:09 msk Exp $";
+static char dkim_filter_c_id[] = "@(#)$Id: dkim-filter.c,v 1.79 2006/09/12 21:46:30 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -1717,15 +1717,7 @@ mlfi_eoh(SMFICTX *ctx)
 
 	dfc->mctx_jobid = smfi_getsymval(ctx, "i");
 	if (dfc->mctx_jobid == NULL)
-	{
-		if (no_i_whine && dolog)
-		{
-			syslog(LOG_WARNING,
-			       "WARNING: sendmail symbol 'i' not available");
-			no_i_whine = FALSE;
-		}
 		dfc->mctx_jobid = JOBIDUNKNOWN;
-	}
 
 #if _FFR_REQUIRED_HEADERS
 	/* if requested, verify RFC2822-required headers */
@@ -2296,6 +2288,26 @@ mlfi_eom(SMFICTX *ctx)
 	dfc = cc->cctx_msg;
 	assert(dfc != NULL);
 
+	/*
+	**  If necessary, try again to get the job ID in case it came down
+	**  later than expected (e.g. postfix).
+	*/
+
+	if (dfc->mctx_jobid == JOBIDUNKNOWN)
+	{
+		dfc->mctx_jobid = smfi_getsymval(ctx, "i");
+		if (dfc->mctx_jobid == NULL)
+		{
+			if (no_i_whine && dolog)
+			{
+				syslog(LOG_WARNING,
+				       "WARNING: sendmail symbol 'i' not available");
+				no_i_whine = FALSE;
+			}
+			dfc->mctx_jobid = JOBIDUNKNOWN;
+		}
+	}
+
 	/* get hostname; used in the X header and in new MIME boundaries */
 	hostname = smfi_getsymval(ctx, "j");
 	if (hostname == NULL)
@@ -2921,13 +2933,16 @@ smfi_getsymval(void *ctx, char *sym)
 int
 dkimf_debug(void)
 {
+	char last;
 	bool done;
+	int crlf = -1;
 	int status;
 	size_t len;
+	int n;
 	time_t now;
 	char *p;
 	char *env[2];
-	char tmphdr[4096];
+	char tmpblock[4096];
 	char data[513];
 	char block[4096];
 	struct sockaddr_in sin;
@@ -2936,7 +2951,7 @@ dkimf_debug(void)
 	srandom(now);
 
 	memset(data, '\0', sizeof data);
-	memset(tmphdr, '\0', sizeof tmphdr);
+	memset(tmpblock, '\0', sizeof tmpblock);
 
 	memset(&sin, '\0', sizeof sin);
 	sin.sin_family = AF_INET;
@@ -3004,31 +3019,31 @@ dkimf_debug(void)
 
 		if (strlen(data) > 0 && isascii(data[0]) && isspace(data[0]))
 		{
-			sm_strlcat(tmphdr, "\r\n", sizeof tmphdr);
-			sm_strlcat(tmphdr, data, sizeof tmphdr);
+			sm_strlcat(tmpblock, "\r\n", sizeof tmpblock);
+			sm_strlcat(tmpblock, data, sizeof tmpblock);
 			continue;
 		}
 
-		if (strlen(tmphdr) != 0)
+		if (strlen(tmpblock) != 0)
 		{
 			char *q;
 
-			p = strchr(tmphdr, ':');
+			p = strchr(tmpblock, ':');
 			*p = '\0';
 			for (q = p + 1; isspace(*q); q++)
 				continue;
-			status = mlfi_header(NULL, tmphdr, q);
+			status = mlfi_header(NULL, tmpblock, q);
 			printf("mlfi_header(NULL, `%s', `%s') returns %s\n",
-			       tmphdr, q, smfis_ret[status]);
+			       tmpblock, q, smfis_ret[status]);
 			if (status != SMFIS_CONTINUE)
 				return 0;
-			memset(tmphdr, '\0', sizeof tmphdr);
+			memset(tmpblock, '\0', sizeof tmpblock);
 		}
 
 		if (strlen(data) == 0)
 			break;
 
-		sm_strlcat(tmphdr, data, sizeof tmphdr);
+		sm_strlcat(tmpblock, data, sizeof tmpblock);
 	}
 
 	status = mlfi_eoh(NULL);
@@ -3037,16 +3052,64 @@ dkimf_debug(void)
 		return 0;
 
 	done = FALSE;
+	last = '\0';
+	n = 0;
+
 	while (!done)
 	{
 		len = fread(block, 1, 4096, stdin);
-		status = mlfi_body(NULL, block, len);
+
+		memset(tmpblock, '\0', sizeof tmpblock);
+
+		for (p = block; p < block + len; p++)
+		{
+			if (n + 2 > sizeof tmpblock)
+			{
+				status = mlfi_body(NULL, tmpblock, n);
+				printf("mlfi_body(NULL, <body>, %d) returns %s\n",
+				       len, smfis_ret[status]);
+
+				if (status != SMFIS_CONTINUE)
+					return 0;
+
+				memset(tmpblock, '\0', sizeof tmpblock);
+				n = 0;
+			}
+
+			if (*p == '\n')
+			{
+				if (crlf == -1)
+				{
+					if (last == '\r')
+						crlf = 1;
+					else
+						crlf = 0;
+				}
+
+				if (crlf == 0)
+				{
+					tmpblock[n] = '\r';
+					n++;
+				}
+			}
+
+			tmpblock[n] = *p;
+			n++;
+
+			last = *p;
+		}
+
+		if (len < 4096)
+			done = TRUE;
+	}
+
+	if (n > 0)
+	{
+		status = mlfi_body(NULL, tmpblock, n);
 		printf("mlfi_body(NULL, <body>, %d) returns %s\n",
 		       len, smfis_ret[status]);
 		if (status != SMFIS_CONTINUE)
 			return 0;
-		if (len < 4096)
-			done = TRUE;
 	}
 
 	status = mlfi_eom(NULL);
@@ -4535,7 +4598,7 @@ main(int argc, char **argv)
 	}
 
 #if _FFR_SELECT_SIGN_HEADERS
-	if (cf != NULL)
+	if (cfg != NULL)
 		(void) config_get(cfg, "SignHeaders", &signhdrs);
 	if (signhdrs != NULL)
 	{
