@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.12 2006/03/10 23:03:49 msk Exp $";
+static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.15 2006/09/15 17:23:33 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -22,24 +22,6 @@ static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.12 2006/03/10 23:03
 #include "dkim.h"
 
 /* lookup tables */
-static struct chartable prv_sigparams[] =	/* signature parameters */
-{
-	{ 'a',		DKIM_PARAM_SIGNALG },
-	{ 'b',		DKIM_PARAM_SIGNATURE },
-	{ 'c',		DKIM_PARAM_CANONALG },
-	{ 'd',		DKIM_PARAM_DOMAIN },
-	{ 'h',		DKIM_PARAM_HDRLIST },
-	{ 'i',		DKIM_PARAM_IDENTITY },
-	{ 'q',		DKIM_PARAM_QUERYMETHOD },
-	{ 's',		DKIM_PARAM_SELECTOR },
-	{ 't',		DKIM_PARAM_TIMESTAMP },
-	{ 'v',		DKIM_PARAM_VERSION },
-	{ 'x',		DKIM_PARAM_EXPIRATION },
-	{ 'z',		DKIM_PARAM_COPIEDHDRS },
-	{ '\0',		-1 }
-};
-struct chartable *sigparams = prv_sigparams;
-
 static struct chartable prv_keyparams[] =	/* key parameters */
 {
 	{ 'a',		DKIM_KEY_ALGORITHM },
@@ -53,6 +35,19 @@ static struct chartable prv_keyparams[] =	/* key parameters */
 };
 struct chartable *keyparams = prv_keyparams;
 
+#ifdef _FFR_ALLMAN_SSP_02
+
+static struct chartable prv_policyparams[] =	/* policy parameters */
+{
+	{ 'p',		DKIM_PPARAM_POLICY },
+	{ 'r',		DKIM_PPARAM_REPORTADDR },
+	{ 't',		DKIM_PPARAM_FLAGS },
+	{ 'u',		DKIM_PPARAM_USERQUERY },
+	{ '\0',		-1 }
+};
+
+#else /* _FFR_ALLMAN_SSP_02 */
+
 static struct chartable prv_policyparams[] =	/* policy parameters */
 {
 	{ 'f',		DKIM_PPARAM_FLAGS },
@@ -62,8 +57,24 @@ static struct chartable prv_policyparams[] =	/* policy parameters */
 	{ 'u',		DKIM_PPARAM_URI },
 	{ '\0',		-1 }
 };
+
+#endif /* _FFR_ALLMAN_SSP_02 */
+
 struct chartable *policyparams = prv_policyparams;
 
+#ifdef _FFR_ALLMAN_SSP_02
+
+static struct nametable prv_policies[] =	/* policies */
+{
+	{ "unknown",	DKIM_POLICY_UNKNOWN },
+	{ "all",	DKIM_POLICY_ALL },
+	{ "strict",	DKIM_POLICY_STRICT },
+	{ '\0',		-1 }
+};
+struct nametable *policies = prv_policies;
+
+#else /* _FFR_ALLMAN_SSP_02 */
+
 static struct chartable prv_policies[] =	/* policies */
 {
 	{ '~',		DKIM_POLICY_SIGNSOME },
@@ -75,6 +86,27 @@ static struct chartable prv_policies[] =	/* policies */
 };
 struct chartable *policies = prv_policies;
 
+#endif /* _FFR_ALLMAN_SSP_02 */
+
+static struct nametable prv_sigparams[] =	/* signature parameters */
+{
+	{ "a",		DKIM_PARAM_SIGNALG },
+	{ "b",		DKIM_PARAM_SIGNATURE },
+	{ "bh",		DKIM_PARAM_BODYHASH },
+	{ "c",		DKIM_PARAM_CANONALG },
+	{ "d",		DKIM_PARAM_DOMAIN },
+	{ "h",		DKIM_PARAM_HDRLIST },
+	{ "i",		DKIM_PARAM_IDENTITY },
+	{ "q",		DKIM_PARAM_QUERYMETHOD },
+	{ "s",		DKIM_PARAM_SELECTOR },
+	{ "t",		DKIM_PARAM_TIMESTAMP },
+	{ "v",		DKIM_PARAM_VERSION },
+	{ "x",		DKIM_PARAM_EXPIRATION },
+	{ "z",		DKIM_PARAM_COPIEDHDRS },
+	{ NULL,		-1 }
+};
+struct nametable *sigparams = prv_sigparams;
+
 static struct nametable prv_algorithms[] =	/* signing algorithms */
 {
 	{ "rsa-sha1",	DKIM_SIGN_RSASHA1 },
