@@ -7,7 +7,7 @@
 #define _DKIM_TABLES_H_
 
 #ifndef lint
-static char dkim_tables_h_id[] = "@(#)$Id: dkim-tables.h,v 1.3 2005/07/11 21:17:37 msk Exp $";
+static char dkim_tables_h_id[] = "@(#)$Id: dkim-tables.h,v 1.5 2006/09/15 05:08:50 msk Exp $";
 #endif /* !lint */
 
 /* structures */
@@ -24,10 +24,14 @@ struct chartable
 };
 
 /* tables */
-struct chartable *sigparams;
 struct chartable *keyparams;
 struct chartable *policyparams;
+
+#ifdef _FFR_ALLMAN_SSP_02
+struct nametable *policies;
+#else /* _FFR_ALLMAN_SSP_02 */
 struct chartable *policies;
+#endif /* _FFR_ALLMAN_SSP_02 */
 
 struct nametable *algorithms;
 struct nametable *canonicalizations;
@@ -35,6 +39,7 @@ struct nametable *hashes;
 struct nametable *keytypes;
 struct nametable *querytypes;
 struct nametable *results;
+struct nametable *sigparams;
 
 /* prototypes */
 extern const char *dkim_code_to_name __P((struct nametable *tbl,
