@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_c_id[] = "@(#)$Id: dkim.c,v 1.164 2006/06/09 06:04:20 msk Exp $";
+static char dkim_c_id[] = "@(#)$Id: dkim.c,v 1.176 2006/09/15 21:23:36 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -72,10 +72,6 @@ static void dkim_mfree(DKIM_LIB *, void *, void *);
 
 #define	DKIM_HDR_SIGNED		0x01
 
-#define	DKIM_CRLF_UNKNOWN	(-1)
-#define	DKIM_CRLF_LF		0
-#define	DKIM_CRLF_CRLF		1
-
 #define	BUFRSZ			1024
 #define	CRLF			"\r\n"
 #define	SP			" "
@@ -675,6 +671,7 @@ dkim_process_set(DKIM *dkim, dkim_set_t type, u_char *str, size_t len)
 		{
 			char addr[MAXADDRESS + 1];
 
+			value = dkim_param_get(set, "d");
 			snprintf(addr, sizeof addr, "@%s", value);
 			dkim->dkim_signer = dkim_strdup(dkim, addr, 0);
 			status = dkim_add_plist(dkim, set, "i",
@@ -1017,8 +1014,9 @@ dkim_sig_signerok(DKIM *dkim, DKIM_SET *set, u_char **hdrs)
 {
 	int status;
 	int c;
+	int clen;
 	struct dkim_header *cur;
-	char *colon;
+	u_char *colon;
 	char *i;
 	char *user;
 	char *domain;
@@ -1033,27 +1031,45 @@ dkim_sig_signerok(DKIM *dkim, DKIM_SET *set, u_char **hdrs)
 
 	assert(i != NULL);
 
+	/* for each header in the "sender header" list */
 	for (c = 0; hdrs[c] != NULL; c++)
 	{
+		/* for each header in the message */
 		for (cur = dkim->dkim_hhead; cur != NULL; cur = cur->hdr_next)
 		{
+			/* skip unsigned headers */
 			if ((cur->hdr_flags & DKIM_HDR_SIGNED) == 0)
 				continue;
 
+			/* determine header name size */
 			colon = strchr(cur->hdr_text, ':');
 			if (colon == NULL)
-				colon = cur->hdr_text;
+				clen = strlen(cur->hdr_text);
+			else
+				clen = colon - cur->hdr_text;
 
-			sm_strlcpy(buf, colon, sizeof buf);
+			/* if this is a sender header */
+			if (strncasecmp(hdrs[c], cur->hdr_text, clen) == 0)
+			{
+				if (colon == NULL)
+					colon = cur->hdr_text;
+				else
+					colon += 1;
 
-			status = rfc2822_mailbox_split(buf, &user, &domain);
-			if (status != 0)
-				continue;
+				sm_strlcpy(buf, colon, sizeof buf);
 
-			snprintf(addr, sizeof addr, "%s@%s", user, domain);
+				status = rfc2822_mailbox_split(buf, &user,
+				                               &domain);
+				if (status != 0)
+					continue;
 
-			if (dkim_addrcmp(addr, i) == 0)
-				return TRUE;
+				snprintf(addr, sizeof addr, "%s@%s",
+				         user, domain);
+
+				/* see if the sender matches "i" */
+				if (dkim_addrcmp(addr, i) == 0)
+					return TRUE;
+			}
 		}
 	}
 
@@ -1062,9 +1078,11 @@ dkim_sig_signerok(DKIM *dkim, DKIM_SET *set, u_char **hdrs)
 
 /*
 **  DKIM_SIG_DOMAINOK -- return TRUE iff a signature appears to have valid
-**                       domain correlation
+**                       domain correlation; that is, "i" must be the same
+**                       domain as or a subdomain of "d"
 **
 **  Parameters:
+**  	dkim -- DKIM handle
 **  	set -- signature set to be checked
 **
 **  Return value:
@@ -1072,13 +1090,14 @@ dkim_sig_signerok(DKIM *dkim, DKIM_SET *set, u_char **hdrs)
 */
 
 static bool
-dkim_sig_domainok(DKIM_SET *set)
+dkim_sig_domainok(DKIM *dkim, DKIM_SET *set)
 {
 	char *at;
 	char *dot;
 	char *i;
 	char *d;
 
+	assert(dkim != NULL);
 	assert(set != NULL);
 	assert(set->set_type == DKIM_SETTYPE_SIGNATURE);
 
@@ -1098,7 +1117,10 @@ dkim_sig_domainok(DKIM_SET *set)
 	for (dot = strchr(at, '.'); dot != NULL; dot = strchr(dot + 1, '.'))
 	{
 		if (strcasecmp(dot + 1, d) == 0)
+		{
+			dkim->dkim_subdomain = TRUE;
 			return TRUE;
+		}
 	}
 
 	return FALSE;
@@ -1647,6 +1669,28 @@ dkim_canonfinal(DKIM *dkim, bool body)
 	}
 }
 
+/*
+**  DKIM_FLUSH_BLANKS -- use accumulated blank lines in canonicalization
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**
+**  Return value:
+**  	None.
+*/
+
+static void
+dkim_flush_blanks(DKIM *dkim)
+{
+	int c;
+
+	assert(dkim != NULL);
+
+	for (c = 0; c < dkim->dkim_blanks; c++)
+		dkim_canonwrite(dkim, CRLF, 2);
+	dkim->dkim_blanks = 0;
+}
+
 /*
 **  DKIM_GENSIGHDR -- generate a signature header
 **
@@ -1903,13 +1947,23 @@ dkim_get_policy(DKIM *dkim)
 	{
 		/* send the NS query */
 		memset(qname, '\0', sizeof qname);
+#ifdef _FFR_ALLMAN_SSP_02
+		if (dkim->dkim_userpolicy)
+		{
+			snprintf(qname, sizeof qname, "%s.%s", dkim->dkim_user,
+			         DKIM_DNSUSERPOLICYNAME);
+		}
+#else /* _FFR_ALLMAN_SSP_02 */
 		snprintf(qname, sizeof qname, "%s.%s", DKIM_DNSPOLICYNAME,
 		         DKIM_DNSKEYNAME);
+#endif /* _FFR_ALLMAN_SSP_02 */
+
 		for (c = n; c < MAXPOLICYDEPTH; c++)
 		{
 			if (label[c] == NULL)
 				continue;
-			sm_strlcat(qname, ".", sizeof qname);
+			if (qname[0] != '\0')
+				sm_strlcat(qname, ".", sizeof qname);
 			sm_strlcat(qname, label[c], sizeof qname);
 		}
 
@@ -1917,9 +1971,15 @@ dkim_get_policy(DKIM *dkim)
 		timeout.tv_sec = dkim->dkim_timeout;
 		timeout.tv_usec = 0;
 
+#ifdef _FFR_ALLMAN_SSP_02
+		q = ar_addquery(ar, qname, C_IN, T_DKIMP, MAXCNAMEDEPTH,
+		                ansbuf, sizeof ansbuf, &arerror,
+		                dkim->dkim_timeout == 0 ? NULL : &timeout);
+#else /* _FFR_ALLMAN_SSP_02 */
 		q = ar_addquery(ar, qname, C_IN, T_TXT, MAXCNAMEDEPTH, ansbuf,
 		                sizeof ansbuf, &arerror,
 		                dkim->dkim_timeout == 0 ? NULL : &timeout);
+#endif /* _FFR_ALLMAN_SSP_02 */
 		if (q == NULL)
 			return -1;
 
@@ -1949,7 +2009,14 @@ dkim_get_policy(DKIM *dkim)
 
 		(void) ar_cancelquery(ar, q);
 #else /* USE_ARLIB */
+
+# ifdef _FFR_ALLMAN_SSP_02
+		status = res_query(qname, C_IN, T_DKIMP, ansbuf,
+		                   sizeof ansbuf);
+# else /* _FFR_ALLMAN_SSP_02 */
 		status = res_query(qname, C_IN, T_TXT, ansbuf, sizeof ansbuf);
+# endif /* _FFR_ALLMAN_SSP_02 */
+
 #endif /* USE_ARLIB */
 
 #if USE_ARLIB
@@ -2086,9 +2153,60 @@ dkim_get_policy(DKIM *dkim)
 	/* decode the payload */
 	status = dkim_process_set(dkim, DKIM_SETTYPE_POLICY, buf, strlen(buf));
 
+#ifdef _FFR_ALLMAN_SSP_02
+	/* if this is a site policy and requires user re-querying, do so */
+	if (!dkim->dkim_userpolicy)
+	{
+		DKIM_SET *s;
+
+		s = dkim_set_first(dkim, DKIM_SETTYPE_POLICY);
+		assert(s != NULL);
+
+		p = dkim_param_get(s, "u");
+		if (p != NULL && strcasecmp(p, "yes") == 0)
+		{
+			dkim->dkim_userpolicy = TRUE;
+			return dkim_get_policy(dkim);
+		}
+	}
+#endif /* _FFR_ALLMAN_SSP_02 */
+
 	return (status == DKIM_STAT_OK ? 0 : -1);
 }
 
+/*
+**  DKIM_POLICY_FLAG -- see if a policy flag is set
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	flag -- flag to check
+**
+**  Return value:
+**  	1 -- flag set
+**  	0 -- flag not set
+**  	-1 -- error
+*/
+
+static int
+dkim_policy_flag(DKIM *dkim, int flag)
+{
+	struct dkim_set *set;
+	u_char *param;
+
+	assert(dkim != NULL);
+	assert(flag != '\0');
+
+	set = dkim_set_first(dkim, DKIM_SETTYPE_POLICY);
+	if (set == NULL)
+		return -1;
+
+	param = dkim_param_get(set, "t");
+	if (param != NULL)
+		return (strchr(param, flag) == NULL) ? 0 : 1;
+	else
+		return 0;
+}
+
 /*
 **  DKIM_GET_KEY -- acquire a public key used for verification
 **
@@ -2219,8 +2337,8 @@ dkim_new(DKIM_LIB *libhandle, const char *id, void *memclosure,
 	new->dkim_tmpdir = libhandle->dkiml_tmpdir;
 	new->dkim_timeout = libhandle->dkiml_timeout;
 	new->dkim_hashtype = DKIM_HASHTYPE_UNKNOWN;
-	new->dkim_crlf = DKIM_CRLF_UNKNOWN;
 	new->dkim_version = DKIM_VERSION_UNKNOWN;
+	new->dkim_blankline = TRUE;
 
 	*statp = DKIM_STAT_OK;
 	return new;
@@ -2899,30 +3017,29 @@ dkim_eoh(DKIM *dkim)
 		     set != NULL;
 		     set = dkim_set_next(set, DKIM_SETTYPE_SIGNATURE))
 		{
-			if (!dkim_sig_versionok(set))
-				break;
-			if (!dkim_sig_domainok(set))
-				break;
-			if (!dkim_sig_signerok(dkim, set,
-			                       dkim->dkim_libhandle->dkiml_senderhdrs))
-				break;
-			if (!dkim_sig_expired(set))
+			/*
+			**  XXX -- move all of this to dkim_eom() to support
+			**         the idea of trying each signature in order
+			**         hoping for one that passes?
+			*/
+
+			hdrlist = dkim_param_get(set, "h");
+			nhdrs = dkim_selecthdrs(dkim, hdrlist, hdrset,
+			                        MAXHDRCNT);
+			if (nhdrs == -1)
+				return DKIM_STAT_INTERNAL;
+
+			if (dkim_sig_versionok(set) &&
+			    dkim_sig_domainok(dkim, set) &&
+			    dkim_sig_signerok(dkim, set,
+			                      dkim->dkim_libhandle->dkiml_senderhdrs) &&
+			    !dkim_sig_expired(set))
 				break;
 		}
+
 		if (set == NULL)
 			return DKIM_STAT_NOSIG;
 
-		/*
-		**  XXX -- move all of this to dkim_eom() to support the idea
-		**         of trying each signature in order hoping for one
-		**         that passes?
-		*/
-
-		hdrlist = dkim_param_get(set, "h");
-		nhdrs = dkim_selecthdrs(dkim, hdrlist, hdrset, MAXHDRCNT);
-		if (nhdrs == -1)
-			return DKIM_STAT_INTERNAL;
-
 		param = dkim_param_get(set, "l");
 		if (param == NULL)
 		{
@@ -3197,7 +3314,6 @@ dkim_eoh(DKIM *dkim)
 DKIM_STAT
 dkim_body(DKIM *dkim, u_char *buf, size_t buflen)
 {
-	int c;
 	size_t wlen;
 	size_t lidx;
 	u_char *p;
@@ -3226,27 +3342,16 @@ dkim_body(DKIM *dkim, u_char *buf, size_t buflen)
 		{
 			if (*p == '\n')
 			{
-
-				/* adaptive */
-				if (dkim->dkim_crlf == DKIM_CRLF_UNKNOWN)
+				if (dkim->dkim_lastchar == '\r')
 				{
-					if (dkim->dkim_lastchar == '\r')
-						dkim->dkim_crlf = DKIM_CRLF_CRLF;
-					else
-						dkim->dkim_crlf = DKIM_CRLF_LF;
-				}
-
-				if ((dkim->dkim_lastchar == '\r' &&
-				     dkim->dkim_crlf == DKIM_CRLF_CRLF) ||
-				    dkim->dkim_crlf == DKIM_CRLF_LF)
-				{
-					if ((dkim->dkim_crlf == DKIM_CRLF_LF &&
-					     wlen == 0) ||
-					    (dkim->dkim_crlf == DKIM_CRLF_CRLF &&
-					     wlen == 1))
+					if (dkim->dkim_blankline)
 					{
 						dkim->dkim_blanks++;
 					}
+					else if (wlen == 1)
+					{
+						dkim_canonwrite(dkim, CRLF, 2);
+					}
 					else
 					{
 						dkim_canonwrite(dkim, wrote,
@@ -3255,58 +3360,59 @@ dkim_body(DKIM *dkim, u_char *buf, size_t buflen)
 
 					wrote = p + 1;
 					wlen = 0;
+					dkim->dkim_blankline = TRUE;
+				}
+			}
+			else if (dkim->dkim_bodycanonalg == DKIM_CANON_RELAXED)
+			{
+				/* non-space after a space */
+				if (dkim_islwsp(dkim->dkim_lastchar) &&
+				    !dkim_islwsp(*p) && *p != '\r')
+				{
+					dkim_flush_blanks(dkim);
+					dkim_canonwrite(dkim, SP, 1);
+					wrote = p;
+					wlen = 1;
+					dkim->dkim_blankline = FALSE;
+				}
+
+				/* space after a non-space */
+				else if (!dkim_islwsp(dkim->dkim_lastchar) &&
+				         dkim_islwsp(*p))
+				{
+					dkim_canonwrite(dkim, wrote, wlen);
+					wlen = 0;
+					wrote = p + 1;
+				}
+
+				/* space after a space */
+				else if (dkim_islwsp(dkim->dkim_lastchar) &&
+				         dkim_islwsp(*p))
+				{
+					/* XXX -- fix logic */
+					;
+				}
+
+				/* non-space after a non-space */
+				else
+				{
+					wlen++;
+					if (dkim->dkim_blankline &&
+					    (wlen > 1 || *p != '\r'))
+					{
+						dkim_flush_blanks(dkim);
+						dkim->dkim_blankline = FALSE;
+					}
 				}
 			}
 			else
 			{
-				if (dkim->dkim_crlf != DKIM_CRLF_CRLF ||
-				    *p != '\r')
+				if (*p != '\r')
 				{
-					for (c = 0; c < dkim->dkim_blanks; c++)
-						dkim_canonwrite(dkim, CRLF, 2);
-					dkim->dkim_blanks = 0;
-				}
-
-				if (dkim->dkim_bodycanonalg == DKIM_CANON_SIMPLE)
-				{
-					wlen++;
-				}
-				else
-				{
-					/* non-space after a space */
-					if (dkim_islwsp(dkim->dkim_lastchar) &&
-					    !dkim_islwsp(*p) &&
-					    (dkim->dkim_crlf != DKIM_CRLF_CRLF ||
-					     *p != '\r'))
-					{
-						dkim_canonwrite(dkim, SP, 1);
-						wrote = p;
-						wlen = 1;
-					}
-
-					/* space after a non-space */
-					else if (!dkim_islwsp(dkim->dkim_lastchar) &&
-					         dkim_islwsp(*p))
-					{
-						dkim_canonwrite(dkim, wrote,
-						                wlen);
-						wlen = 0;
-					}
-
-					/* space after a space */
-					else if (dkim_islwsp(dkim->dkim_lastchar) &&
-					         dkim_islwsp(*p))
-					{
-						/* XXX -- fix logic */
-						;
-					}
-
-					/* non-space after a non-space */
-					else
-					{
-						wlen++;
-					}
+					dkim_flush_blanks(dkim);
+					dkim->dkim_blankline = FALSE;
 				}
+				wlen++;
 			}
 
 			dkim->dkim_lastchar = *p;
@@ -3356,6 +3462,9 @@ dkim_body(DKIM *dkim, u_char *buf, size_t buflen)
 **
 **  Return value:
 **  	A DKIM_STAT_* constant.
+**
+**  XXX -- consider removing "testkey", and having the caller use
+**         dkim_policy() retrieve flags like that one instead
 */
 
 DKIM_STAT
@@ -3676,13 +3785,16 @@ dkim_eom(DKIM *dkim, bool *testkey)
 		{
 			int status;
 			u_int keyalg;
-			size_t hashlen;
+			size_t bhashlen = 0;
+			size_t hashlen = 0;
 			BIO *key = NULL;
 			u_char *keytype;
 			u_char *hashtypes;
 			u_char *keygran;
 			u_char *test;
-			void *hashout;
+			u_char *bhash;
+			void *hashout = NULL;
+			void *bhashout = NULL;
 			struct dkim_set *keyset;
 
 			msgsigned = TRUE;
@@ -3732,6 +3844,7 @@ dkim_eom(DKIM *dkim, bool *testkey)
 			if (!dkim_key_hashok(dkim, hashtypes))
 				return DKIM_STAT_CANTVRFY;
 
+			/* key granularity is OK? */
 			keygran = dkim_param_get(keyset, "g");
 			if (keygran != NULL && !dkim_key_granok(dkim, keygran))
 				return DKIM_STAT_CANTVRFY;
@@ -3780,6 +3893,14 @@ dkim_eom(DKIM *dkim, bool *testkey)
 				hashout = sha1->sha1_out;
 				hashlen = SHA_DIGEST_LENGTH;
 
+				if (dkim->dkim_bhash != NULL)
+				{
+					sha1 = (struct dkim_sha1 *) dkim->dkim_bhash;
+
+					bhashout = sha1->sha1_out;
+					bhashlen = SHA_DIGEST_LENGTH;
+				}
+
 				break;
 			  }
 
@@ -3793,6 +3914,14 @@ dkim_eom(DKIM *dkim, bool *testkey)
 				hashout = sha256->sha256_out;
 				hashlen = SHA256_DIGEST_LENGTH;
 
+				if (dkim->dkim_bhash != NULL)
+				{
+					sha256 = (struct dkim_sha256 *) dkim->dkim_bhash;
+
+					bhashout = sha256->sha256_out;
+					bhashlen = SHA256_DIGEST_LENGTH;
+				}
+
 				break;
 			  }
 #endif /* SHA256_DIGEST_LENGTH */
@@ -3801,6 +3930,21 @@ dkim_eom(DKIM *dkim, bool *testkey)
 				assert(0);
 			}
 
+			/* body hash, if present, matches this body? */
+			bhash = dkim_param_get(sigset, "bh");
+			if (bhash != NULL && bhashout != NULL)
+			{
+				char b64hash[MAXHEADER + 1];
+
+				memset(b64hash, '\0', sizeof b64hash);
+				status = dkim_base64_encode(bhashout, bhashlen,
+				                            b64hash,
+				                            sizeof b64hash);
+
+				if (strcmp(b64hash, bhash) != 0)
+					return DKIM_STAT_BADSIG;
+			}
+
 			switch (dkim->dkim_sigtype)
 			{
 			  case DKIM_SIGTYPE_RSA:
@@ -3875,9 +4019,38 @@ dkim_eom(DKIM *dkim, bool *testkey)
 
 		ret = DKIM_STAT_OK;
 		policy = dkim_policy(dkim, &test);
+		if (testkey != NULL)
+			*testkey = test;
 
+		/*
+		**  If the "s" policy flag is set and the signature claims
+		**  the "i=" domain is a subdomain of the "d=" domain,
+		**  the signature is not acceptable.
+		*/
+
+		if (dkim_policy_flag(dkim, 's') && dkim->dkim_subdomain)
+			return DKIM_STAT_CANTVRFY;
+
+		/* react to the published policy */
 		switch (policy)
 		{
+#ifdef _FFR_ALLMAN_SSP_02
+
+		  case DKIM_POLICY_UNKNOWN:
+			break;
+
+		  case DKIM_POLICY_ALL:
+			if (!msgsigned)
+				ret = DKIM_STAT_NOSIG;
+			break;
+
+		  case DKIM_POLICY_STRICT:
+			if (!verified)
+				ret = DKIM_STAT_BADSIG;
+			break;
+
+#else /* _FFR_ALLMAN_SSP_02 */
+
 		  case DKIM_POLICY_SIGNSOME:
 			break;
 
@@ -3899,6 +4072,8 @@ dkim_eom(DKIM *dkim, bool *testkey)
 			ret = DKIM_STAT_NOTIMPLEMENT;
 			break;
 
+#endif /* _FFR_ALLMAN_SSP_02 */
+
 		  default:
 			ret = DKIM_STAT_SYNTAX;
 			break;
@@ -4273,7 +4448,7 @@ dkim_reportinfo(DKIM *dkim, int *fd, int *bfd, u_char *buf, size_t buflen)
 }
 
 /*
-**  DKIM_POLICY -- retreive policy associated with the sender's domain
+**  DKIM_POLICY -- parse policy associated with the sender's domain
 **
 **  Parameters:
 **  	dkim -- DKIM handle
@@ -4287,6 +4462,9 @@ dkim_policy_t
 dkim_policy(DKIM *dkim, bool *test)
 {
 	struct dkim_set *set;
+#ifdef _FFR_ALLMAN_SSP_02
+	struct dkim_set *uset;
+#endif /* _FFR_ALLMAN_SSP_02 */
 	u_char *param;
 
 	assert(dkim != NULL);
@@ -4296,10 +4474,16 @@ dkim_policy(DKIM *dkim, bool *test)
 	if (set == NULL)
 		return -1;
 
+#ifdef _FFR_ALLMAN_SSP_02
+	uset = dkim_set_next(set, DKIM_SETTYPE_POLICY);
+	if (uset != NULL)
+		set = uset;
+#endif /* _FFR_ALLMAN_SSP_02 */
+
 	param = dkim_param_get(set, "t");
 	if (param != NULL)
 	{
-		if (param[0] == 'y')
+		if (strchr(param, 'y') != NULL)
 			*test = TRUE;
 		else
 			*test = FALSE;
@@ -4309,11 +4493,19 @@ dkim_policy(DKIM *dkim, bool *test)
 		*test = FALSE;
 	}
 
+#ifdef _FFR_ALLMAN_SSP_02
+	param = dkim_param_get(set, "p");
+	if (param == NULL)
+		return -1;
+
+	return (dkim_policy_t) dkim_name_to_code(policies, param);
+#else /* _FFR_ALLMAN_SSP_02 */
 	param = dkim_param_get(set, "o");
 	if (param == NULL)
 		return -1;
 
 	return (dkim_policy_t) dkim_char_to_code(policies, param[0]);
+#endif /* _FFR_ALLMAN_SSP_02 */
 }
 
 /*
