@@ -7,7 +7,7 @@
 #define _DKIM_H_
 
 #ifndef lint
-static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.41 2006/06/09 06:03:45 msk Exp $";
+static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.48 2006/09/15 23:50:20 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -47,7 +47,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.41 2006/06/09 06:03:45 msk Exp $"
 #define MAXADDRESS		256	/* biggest user@host we accept */
 #define MAXCNAMEDEPTH		3	/* max. CNAME recursion we allow */
 #define MAXHDRCNT		64	/* max. headers signed */
-#define MAXHEADER		1000	/* buffer for caching one header */
+#define MAXHEADER		4096	/* buffer for caching one header */
 #define MAXHEADERS		32768	/* buffer for caching headers */
 #define MAXPOLICYDEPTH		5	/* policy recursion we allow */
 #define MAXTAGNAME		8	/* biggest tag name */
@@ -69,6 +69,10 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.41 2006/06/09 06:03:45 msk Exp $"
 					/* reserved DNS sub-zone */
 #define	DKIM_DNSPOLICYNAME	"_policy"
 					/* reserved DNS sub-zone */
+#ifdef _FFR_ALLMAN_SSP_02
+# define DKIM_DNSUSERPOLICYNAME	"_user"
+					/* reserved DNS sub-zone */
+#endif /* _FFR_ALLMAN_SSP_02 */
 
 /*
 **  DKIM_VERSION -- version; attempts to support backward compatibility
@@ -159,6 +163,7 @@ typedef int dkim_param_t;
 #define DKIM_PARAM_TIMESTAMP	9	/* t */
 #define DKIM_PARAM_EXPIRATION	10	/* x */
 #define DKIM_PARAM_COPIEDHDRS	11	/* z */
+#define DKIM_PARAM_BODYHASH	12	/* bh */
 
 /*
 **  DKIM_KEY -- known key parameters
@@ -180,11 +185,22 @@ typedef int dkim_key_t;
 
 typedef int dkim_pparam_t;
 
-#define	DKIM_PPARAM_POLICY	0	/* o */
-#define	DKIM_PPARAM_FLAGS	1	/* f */
-#define	DKIM_PPARAM_NOTES	2	/* n */
-#define	DKIM_PPARAM_REPORTADDR	3	/* r */
-#define	DKIM_PPARAM_URI		4	/* u */
+#ifdef _FFR_ALLMAN_SSP_02
+
+# define DKIM_PPARAM_POLICY	0	/* p */
+# define DKIM_PPARAM_REPORTADDR	1	/* r */
+# define DKIM_PPARAM_FLAGS	2	/* t */
+# define DKIM_PPARAM_USERQUERY	3	/* u */
+
+#else /* _FFR_ALLMAN_SSP_02 */
+
+# define DKIM_PPARAM_POLICY	0	/* o */
+# define DKIM_PPARAM_FLAGS	1	/* f */
+# define DKIM_PPARAM_NOTES	2	/* n */
+# define DKIM_PPARAM_REPORTADDR	3	/* r */
+# define DKIM_PPARAM_URI	4	/* u */
+
+#endif /* _FFR_ALLMAN_SSP_02 */
 
 /*
 **  DKIM_POLICY -- policies
@@ -192,14 +208,27 @@ typedef int dkim_pparam_t;
 
 typedef int dkim_policy_t;
 
-#define	DKIM_POLICY_SIGNSOME	0	/* ~ */
-#define	DKIM_POLICY_REQUIRED	1	/* - */
-#define	DKIM_POLICY_AUTHORITY	2	/* ! */
-#define	DKIM_POLICY_NEVER	3	/* . */
-#define	DKIM_POLICY_USER	4	/* ^ */
+#ifdef _FFR_ALLMAN_SSP_02
 
-#define	DKIM_POLICY_DEFAULT	DKIM_POLICY_SIGNSOME
-#define	DKIM_POLICY_DEFAULTTXT	"o=~"
+# define DKIM_POLICY_UNKNOWN	0	/* unknown */
+# define DKIM_POLICY_ALL	1	/* all */
+# define DKIM_POLICY_STRICT	2	/* strict */
+
+# define DKIM_POLICY_DEFAULT	DKIM_POLICY_UNKNOWN
+# define DKIM_POLICY_DEFAULTTXT	"p=unknown; u=no"
+
+#else /* _FFR_ALLMAN_SSP_02 */
+
+# define DKIM_POLICY_SIGNSOME	0	/* ~ */
+# define DKIM_POLICY_REQUIRED	1	/* - */
+# define DKIM_POLICY_AUTHORITY	2	/* ! */
+# define DKIM_POLICY_NEVER	3	/* . */
+# define DKIM_POLICY_USER	4	/* ^ */
+
+# define DKIM_POLICY_DEFAULT	DKIM_POLICY_SIGNSOME
+# define DKIM_POLICY_DEFAULTTXT	"o=~"
+
+#endif /* _FFR_ALLMAN_SSP_02 */
 
 /*
 **  DKIM_SETTYPE -- types of sets
@@ -630,5 +659,4 @@ extern DKIM_STAT dkim_free __P((DKIM *dkim));
 
 extern int rfc2822_mailbox_split __P((char *addr, char **user, char **domain));
 
-
 #endif /* ! _DKIM_H_ */
